;FLASH
;-----
;by Levi Lansing
;--puzzlesome@hotmail.com
;--http://www.crosswinds.net/levisti86/

;by the way, this was made in Assembly Studio 86 v3.0 so if you have v3.1, you
;may have to change the header files...

#include "asm86.h"
#include "ti86asm.inc"

_user_int_ram EQU $d2fd
_off  equ  4101h

.org _asm_exec_ram
  bit 2,(iy+$23)	;is the user interrupt on?
  jr nz,undo		;go turn it it off if it is
   res 2,(iy+$23)    ;turn user int off so it won't get called by
                     ;accident before we're ready
   
  call _clrScrn		;clear the screen
  call _homeup		;put the cursor at the top of the screen
  ld hl,TEXT		;point hl at the text
  call _puts		;put the text on the screen

   ld hl,int         		;hl point to the begining of the program
   ld de,_user_int_ram+1	;de pionts to where the program is to be loaded
   ld bc,int_end-int		;bc is the length of the program to be sent
   ldir						;copies it
   
   ld a,(_user_int_ram+1)		;this sets up checksum byte
   ld hl,_user_int_ram+($28*1)	;we need to add the 1st bite with one
   add a,(hl)					;every 40 bytes and store it in the...
   ld hl,_user_int_ram+($28*2)
   add a,(hl)
   ld hl,_user_int_ram+($28*3)
   add a,(hl)
   ld hl,_user_int_ram+($28*4)
   add a,(hl)
   ld hl,_user_int_ram+($28*5)
   add a,(hl)
   ld (_user_int_ram),a			;first byte of the interrupt area

   call _getkey
   set 2,(iy+$23)	;turn on the interrupt
   ret				;return from the program

undo:
  res 2,(iy+$23)					;turn off the
  res textInverse,(iy+textflags)	;set the text to normal (turn of the inverse)
  call _clrScrn						;clear the screen
  ret								;return from the program

TEXT:	;just the text to be displayed (21 spaces per line)
	.db "2nd+MODE+F1 =spd up  "
	.db "2nd+MODE+F2 =spd down"
	.db "Press any key...     ",0

int:
   push af			;when you run an interrupt, you can't change ANY registers
   push bc			;so you must push all the regesters you use
   push hl			;and pop (restore) them at the end

  ld a,%00111111	;key port mask
  out (1),a			;send it to the key port
  in a,(1)			;get it back
  cp 79				;if the returned value = 79...
  jr z,UP			;you are holding 2nd, MODE, and F1, so we will speed it up
  cp 87				;if the returned value = 87
  jr z,DOWN			;you are holding 2nd, MODE, and F2, so we will decrease the speed
cont:
  ld hl,var			;var is the counter which creates the delay
  dec (hl)			;it's decreased by one everythime the interupt occurs
  ld a,(hl)			;and then
  cp 0				;if it is not 0
  jr nz,done		;we will quit the interrupt
  ld a,(Speed)		;if the counter has counted down
  ld (hl),a			;restore it with the speed
  ld hl,var+1		;the second variable is either a 1 or a 0...
  ld a,(hl)			;it tells the program wheither to inverse the text, or not
  cp 1					;if it's NOT 1
  jr nz,inverse_text1	;we will inverse the text
  res textInverse,(iy+textflags)	;if it is 1, then make the text normal
  ld (hl),0							;and change the var back to 0
  jr inverse_screen					;now inverse the screen
inverse_text1:
  set textInverse,(iy+textflags)	;set the text flag to write inversely
  ld (hl),1							;change the var to 1
inverse_screen:
  ld hl,$FC00		;point hl to the begining of the screen memory
  ld bc,$8006		;b will be 128 and c will be 8 (we will loop 
Loop:
  ld a,$FF		;%11111111 in binary, when this is xor'd to the screen
  xor (hl)		;it will inverse the data on the screen
  ld (hl),a		;load the inversed data back into the screen
  inc hl		;increase hl to the next byte in the screen data
  djnz Loop		;dec b and if it's not zero, Loop
  dec b			;b will be decreased from 0 to 255 ($FF)
  dec c			;decrement c 
  ld a,c		;load it into a
  cp 0			;test if it's 0 and if it is
  jr z,done		;we're done
  jr Loop		;loop if it's not 0
done:
   pop hl		;restore our variables in the opposide order we pushed them
   pop bc
   pop af
   ret			;return from the interrupt

UP:
  ld a,(Speed)	;value of speed into a
  dec a			;subtract 1
  cp 6			;if it's 6...
  jr z,cont		;don't save it (after 6 it goes too fast!)
  ld (Speed),a	;put a back in the speed
  ld a,(var)
  dec a			;decrease the counter
  cp 0
  jr z,cont		;if it's zero we won't put it back in and continue with the counting down
  ld (var),a	;if it isn't 0, store it back in var
  jr cont		;continue counting down

DOWN:
  ld a,(Speed)	;do the same stuff as the up routine...
  inc a			;only increase instead of decreasing...
  cp 0
  jr z,cont
  ld (Speed),a
  ld a,(var)
  inc a
  cp 0			;it will be 0 if it was 255 and increased by 1
  jr z,cont		;if it is, continue with the counting down
  ld (var),a	;put a back
  jr cont		;continue


var equ $-int+_user_int_ram+1
  .db $FF,0
Speed equ $-int+_user_int_ram+3
  .db $25
int_end:

.end		;All DONE!

